@@ -1,5 +1,5 @@
 #  SEMgraph library
-#  Copyright (C) 2019-2021 Fernando Palluzzi; Mario Grassi
+#  Copyright (C) 2019-2021 Mario Grassi; Fernando Palluzzi 
 #  e-mail: <fernando.palluzzi@gmail.com>
 #  University of Pavia, Department of Brain and Behavioral Sciences
 #  Via Bassi 21, 27100 Pavia, Italy
@@ -19,19 +19,14 @@
 
 # -------------------------------------------------------------------- #
 
-#' @title SEM-based gene set analysis
+#' @title SEM-based gene set analysis (GSA)
 #'
 #' @description Gene Set Analysis (GSA) via self-contained test for group
-#' effect on signaling (directed) pathways as SEM, evaluating overall pathway
-#' perturbation, perturbation emission from source nodes, and perturbation
-#' accumulation on target nodes. Approximate randomization test P-values
-#' of specific node and aggregated group effects will be computed.
-#' For directed graphs, they include: the sum of group effects adjusted
-#' by residual variances (D), the sum of the tagret nodes perturbation
-#' (i.e., group effect) accumulation from source nodes (A), and the sum
-#' of the source nodes perturbation emission towards target nodes (E).
-#' For undirected graphs, the sum of group effects (D), adjusted
-#' by residual variances, will be estimated.
+#' effect on signaling (directed) pathways based on SEM. The core of the
+#' methodology is implemented in the RICF algorithm of \code{SEMrun()},
+#' recovering from RICF output node-specific group effect p-values, and
+#' Brown’s combined permutation p-values of node activation and inhibition.
+#'
 #' @param g A list of pathways to be tested.
 #' @param data A matrix or data.frame. Rows correspond to subjects, and
 #' columns to graph nodes (variables).
@@ -45,23 +40,44 @@
 #' @param n_rep Number of randomization replicates (default = 1000).
 #' @param ... Currently ignored.
 #'
+#' @details For gaining more biological insights into the functional roles
+#' of pre-defined subsets of genes, node perturbation obtained from RICF
+#' fitting has been combined with up- or down-regulation of genes from KEGG
+#' to obtain overall pathway perturbation as follows: 
+#' \itemize{
+#' \item The node perturbation is defined as activated when the minimum among
+#' the p-values is positive; if negative, the status is inhibited. 
+#' \item Up- or down- regulation of genes (derived from KEGG database) has
+#' been obtained from the weighted adjacency matrix of each pathway as column
+#' sum of weights over each source node. If the overall sum of node weights
+#' is below 1, the pathway is flagged as down-regulated otherwise as up-regulated. 
+#' \item The combination between these two quantities allows to define the
+#' direction (up or down) of gene perturbation. Up- or down regulated gene status,
+#' associated with node inhibition, indicates a decrease in activation (or
+#' increase in inhibition) in cases with respect to control group. Conversely,
+#' up- or down regulated gene status, associated with node activation, indicates
+#' an increase in activation (or decrease in inhibition) in cases with
+#' respect to control group.
+#' }
+#' 
 #' @return A list of 2 objects:
 #' \enumerate{
 #' \item "gsa", A data.frame reporting the following information for each
 #' pathway in the input list:
 #' \itemize{
-#' \item "N.nodes", pathway size (number of nodes);
-#' \item "N.DRNs", number of differential regulated genes (DRNs) within the pathway,
-#' after multiple test correction with Benjamini-Hochberg method;
-#' \item "pD", significance of the sum of group effects, adjusted by the
-#' residual variance;
-#' \item "pA", significance of the sum of tagret nodes perturbation
-#' (i.e., group effect) accumulation from source nodes;
-#' \item "pE", significance of the sum of source nodes perturbation
-#' (i.e., group effect) emission towards target nodes;
-#' \item "pvalue", Fisher's combined P-value of pD, pA, and pE.
+#' \item "No.nodes", pathway size (number of nodes);
+#' \item "No.DEGs", number of differential espression genes (DEGs) within
+#' the pathway, after multiple test correction with one of the methods
+#' available in \code{\link[stats]{p.adjust}};
+#' \item "pert", pathway perturbation status (see details);
+#' \item "pNA", Brown's combined P-value of pathway node activation;
+#' \item "pNI", Brown's combined P-value of pathway node inhibition;
+#' \item "PVAL", Bonferroni combined P-value of pNA, and pNI; i.e.,
+#' 2* min(pNA, PNI);
+#' \item "ADJP", Adjusted Bonferroni P-value of pathway perturbation; i.e.,
+#' min(No.pathways * PVAL; 1).
 #' }
-#' \item "DRN", a list with DRNs names per pathways.
+#' \item "DEG", a list with DEGs names per pathways.
 #' }
 #'
 #' @import igraph
@@ -76,11 +92,11 @@
 #' \dontrun{
 #'
 #' # Nonparanormal(npn) transformation
-
+#'
 #' library(huge)
 #' als.npn <- huge.npn(alsData$exprs)
 #'
-#' # Selection of FTD pathways from kegg.pathways.Rdata
+#' # Selection of FTD-ALS pathways from kegg.pathways.Rdata
 #'
 #' paths.name <- c("MAPK signaling pathway",
 #'                 "Protein processing in endoplasmic reticulum",
@@ -95,73 +111,162 @@
 #'               method = "bonferroni", alpha = 0.05,
 #'               n_rep = 1000)
 #' GSA$gsa
-#' GSA$DRN
+#' GSA$DEG
 #' 
 #' }
 #'
-SEMgsa <- function(g = list(), data, group, method = "BH", alpha = 0.05,
-                   n_rep = 1000, ...)
+SEMgsa<- function(g=list(), data, group, method = "BH", alpha = 0.05, n_rep = 1000, ...)
 {
-	# Set SEM objects
-	pX2 <- function(x) 1 - pchisq(-2*sum(log(x)), 2*length(x))
+ 	# set loop objects:
 	gs <- names(g)
-	K <- length(g)
-	del <- NULL
+    K <- length(g)
 	res.tbl <- NULL
-	DRN <- list()
-
-	for (k in 1:K) {
-		cat(paste("# ", k, ": ", gs[k], "\n", sep = ""))
-		quiet(ig <- properties(g[[k]])[[1]])
-
-		# SEM fitting
-		fit <- NULL
-		err <- paste(" ValueError: none of pathway #", k,
-		             " variables are present in the dataset.\n",
-		             sep = "")
-		tryCatch(quiet(fit <- SEMricf(graph = ig, data, group,
-		                              random.x = FALSE,
-	                                  n_rep)),
-	             error = function(c) cat(err))
-
-		if (length(fit[[1]]) == 0) {
-			del <- c(del, k)
-			next
-		}
-
-		p <- ncol(fit$dataXY)
-		B <- (diag(p) - fit$fit$ricf$Bhat)[-1, -1]
-		if (sum(B) == 0) {
-			del <- c(del, k)
-			next
-		}
-
-		pval <- fit$gest$pvalue[-c(1:3)]
-		genes <- gsub("X", "", rownames(fit$gest))[-c(1:3)]
-		genes <- genes[p.adjust(pval, method = method) < alpha]
-		DRN <- c(DRN, list(genes))
-
-		# data.frame of SEM results
-		df <- data.frame(
-		 N.nodes = vcount(ig),
-		 #N.edges= ecount(ig),
-		 N.DRNs = sum(p.adjust(pval, method = method) < alpha),
-		 pD = round(fit$gest[1, 4], 6),
-		 pA = round(fit$gest[2, 4], 6),
-		 pE = round(fit$gest[3, 4], 6))
-		pvalue = round(pX2(x = df[1, 3:5]), 6)
-		res.tbl <- rbind(res.tbl, cbind(df, pvalue))
+	DEG <- list()
+		
+	for (k in 1:K){ #k=1
+	 cat( "k =", k, gs[k], "\n" )
+	 ig <- simplify(g[[k]], remove.loops = TRUE)
+	 adj <- as.matrix(get.adjacency(ig, attr = "weight"))
+	 adj <- colSums(adj)
+	 nodes <- ifelse(adj >= 1, 1, ifelse(adj == 0, 0, -1)) #table(nodes)
+	 status <- ifelse(sum(nodes) >= 1, 1, ifelse(sum(nodes) == 0, 0, -1))
+	 
+	 # RICF fitting:
+	 fit <- NULL
+	 err <- paste(" ValueError: Model converged = FALSE for k =",k,"\n")
+	 tryCatch(fit <- quiet(SEMricf(
+	 					 ig, data, group, random.x = FALSE, n_rep)),
+	 					 error = function(c) cat(err))
+	 if (length(fit[[1]]) == 0) {
+	  res.tbl<- rbind(res.tbl, rep(NA,6))
+	  colnames(res.tbl) <- c("No.nodes","No.DEGs","pert","pNa","pNi","PVAL")
+	  next
+	 }
+	 pval<- fit$gest$pvalue
+	 genes<- gsub("X", "", rownames(fit$gest))
+	 genes<- genes[p.adjust(pval, method=method) < alpha]
+	 DEG<- c(DEG, list(genes))
+	 
+	 # data.frame of combined SEM results :
+	 cpval <- fit$pval
+	 names(cpval) <- c("pNa", "pNi")
+	 pvmin <- names(cpval)[which.min(cpval)]
+	 sign <- ifelse(pvmin == "pNa", "+", "-")
+	 if ( status != 0 ) {
+	  if (status == -1 & sign == "-") pert <- "up inh"
+	  else if (status == -1 & sign == "+") pert <- "down inh"
+	  else if (status == 1 & sign == "+") pert <- "up act"
+	  else if (status == 1 & sign == "-") pert <- "down act"
+	 }else{pert <- NA}
+	 df <- data.frame(
+	   No.nodes = vcount(ig),
+	   No.DEGs = sum(p.adjust(pval, method=method) < alpha),
+	   pert = pert,
+	   pNa = cpval[1],
+	   pNi = cpval[2],
+	   PVAL = 2 * min(cpval[1], cpval[2]))
+	 res.tbl <- rbind(res.tbl,df)
 	}
+ 
+ 	ADJP <- p.adjust(res.tbl$PVAL, method="bonferroni")
+	res.tbl<- cbind(res.tbl, ADJP)
+	rownames(res.tbl) <- names(DEG) <- names(g)
+	gsa <- res.tbl[order(res.tbl$ADJP),]
+	DEG <- DEG[rownames(gsa)]
+	
+	return( list(gsa=gsa, DEG=DEG) )
+}
 
-	if(is.null(del)) {
-		rownames(res.tbl) <- gs
-		names(DRN) <- gs
-	} else {
-		rownames(res.tbl)<- gs[-del]
-		names(DRN)<- gs[-del]
+#' @title SEM-based differential connectivity inference (DCI)
+#'
+#' @description Creates a network with perturbed edges obtained from 
+#' the output of \code{\link[SEMgraph]{SEMrun}} with two-group and CGGM solver.
+#' To increase the efficiency of computations for large graphs,
+#' users can select to break the network structure into clusters and select
+#' the topological clustering method (see \code{\link[SEMgraph]{clusterGraph}}). 
+#' The function \code{\link[SEMgraph]{SEMrun}} is applied iteratively on 
+#' each cluster to obtain the graph with the full list of perturbed edges. 
+#' 
+#' @param graph Input network as an igraph object.
+#' @param data A matrix or data.frame. Rows correspond to subjects, and
+#' columns to graph nodes (variables).
+#' @param group A binary vector. This vector must be as long as the number
+#' of subjects. Each vector element must be 1 for cases and 0 for control
+#' subjects.
+#' @param C Topological clustering methods (default C = "none"). If
+#' \code{type = "tahc"}, network modules are generated using the tree
+#' agglomerative hierarchical clustering method (Yu et al., 2015).
+#' Other non-tree clustering methods from igraph package include: "wtc"
+#' (walktrap community structure with short random walks),
+#' "ebc" (edge betweeness clustering), "fgc" (fast greedy method), "lbc"
+#' (label propagation method), "lec" (leading eigenvector method), "loc"
+#' (multi-level optimization), "opc" (optimal community structure), "sgc"
+#' (spinglass statistical mechanics).
+#' @param method Multiple testing correction method. One of the values
+#' available in \code{\link[stats]{p.adjust}}. By default, method is set
+#' to "BH" (i.e., Benjamini-Hochberg correction).
+#' @param alpha Gene set test significance level (default = 0.05).
+#' @param ... Currently ignored.
+#'
+#' @return An igraph object.
+#'
+#' @import igraph
+#' @import GGMncv
+#' @export
+#'
+#' @author Mario Grassi \email{mario.grassi@unipv.it}
+#'
+#' @examples
+#'
+#' \dontrun{
+#'
+#' # Nonparanormal(npn) transformation
+#' library(huge)
+#' als.npn <- huge.npn(alsData$exprs)
+#'
+# Extract the "MAPK signaling pathway"
+#' g <- kegg.pathways[["MAPK signaling pathway"]]
+#' G <- properties(g)[[1]]; summary(G)
+#'
+#' # Create ALS network with perturbed edges using edge betweeness clustering
+#' gU<- SEMdci(G, als.npn, alsData$group, C="ebc", method="BH", alpha=0.2)
+#' gcU<- properties(gU)
+#'
+#' old.par <- par(no.readonly = TRUE)
+#' par(mfrow=c(2,2), mar=rep(1,4))
+#' gplot(gcU[[1]], l="fdp") # max component
+#' gplot(gcU[[2]], l="fdp") # 2nd cpmponent
+#' gplot(gcU[[3]], l="fdp") # 3rd component 
+#' gplot(gcU[[4]], l="fdp") # 4th component
+#' par(old.par)
+#'
+#' }
+#'
+SEMdci<- function(graph, data, group, C = "none", method = "BH", alpha = 0.05, ...)
+{
+	if (C != "none"){
+	 C <- clusterGraph(graph, type = C, size = 10)#table(C)
+	 K <- as.numeric(names(table(C)))
+	 gL <- NULL
+	 for (k in K){ #k=1
+	  cat( "fit cluster =", k, "\n" )
+	  g <- induced_subgraph(graph, vids= names(C)[C == k])
+	  dest <- quiet(SEMggm2(g, data, group, method, alpha)$dest)
+	  dsub <- subset(dest, dest$pvalue < alpha)
+	  ftm <- data.frame(from=dsub$rhs, to=dsub$lhs)
+	  gC <- graph_from_data_frame(ftm) #gplot(gU)
+	  if (ecount(gC) > 0) gL <- c(gL,list(gC))
+	 }
+	cat("Done.\n")
+	gU <- graph.union(gL)
+	}else{
+	 g <- graph
+	 dest <- quiet(SEMggm2(g, data, group, method, alpha)$dest)
+	 dsub <- subset(dest, dest$pvalue < alpha)
+	 ftm <- data.frame(from=dsub$rhs, to=dsub$lhs)
+	 gU <- graph_from_data_frame(ftm) #gplot(gU)
 	}
-
-	return(list(gsa = res.tbl, DRN = DRN))
+	return(gU)
 }
 
 #' @title Graph properties summary and graph decomposition
@@ -169,9 +274,9 @@ SEMgsa <- function(g = list(), data, group, method = "BH", alpha = 0.05,
 #' @description Produces a summary of network properties and returns
 #' graph components (ordered by decreasing size), without self-loops.
 #' @param graph Input network as an igraph object.
-#' @param data An optional data matrix whith rows corresponding to subjects,
-#' and columns to graph nodes (variables). Nodes will be mapped onto
-#' variable names.
+#' @param data An optional data matrix (default data = NULL) whith rows
+#' corresponding to subjects, and columns to graph nodes (variables).
+#' Nodes will be mapped onto variable names. 
 #' @param ... Currently ignored.
 #'
 #' @import igraph
@@ -186,74 +291,37 @@ SEMgsa <- function(g = list(), data, group, method = "BH", alpha = 0.05,
 #'
 #' # Extract the "Type II diabetes mellitus" pathway:
 #' g <- kegg.pathways[["Type II diabetes mellitus"]]
-#' properties(g)
 #' summary(g)
+#' properties(g)
 #'
-properties <- function(graph, data = NULL, ...)
+properties<- function (graph, data = NULL, ...) 
 {
-    if (!is_igraph(graph)) ig <- graph_from_graphnel(graph) else ig <- graph
-    if (!is.null(data)) {
-		nodes <- colnames(data)[colnames(data) %in% V(graph)$name]
-		ig <- induced_subgraph(graph, vids = which(V(graph)$name %in% nodes))
-	}
-	ig <- simplify(ig, remove.loops = TRUE)
-	gcs <- igraph::decompose.graph(ig)
-	vsize <- lapply(1:length(gcs), function(x) vcount(gcs[[x]]))
-	ig1 <- gcs[[which.max(vsize)]]
+    if (!is_igraph(graph)){
+     ig <- graph_from_graphnel(graph)
+    }else{ ig <- graph }
+    if (!is.null(data)){
+     nodes <- colnames(data)[colnames(data) %in% V(graph)$name]
+     ig <- induced_subgraph(graph, vids = which(V(graph)$name %in% nodes))
+    }
+    
+	ig <- igraph::simplify(ig, remove.loops = TRUE)
+    gcs <- igraph::decompose.graph(ig, min.vertices = 2)
+    vsize <- sapply(1:length(gcs), function(x) vcount(gcs[[x]]))
+	names(vsize) <- 1:length(vsize)
+	gcs <- gcs[as.numeric(names(sort(vsize, decreasing = TRUE)))]
+    
 	cat("Frequency distribution of graph components\n\n")
-	tab <- table(sapply(gcs, vcount))
-	tab <- data.frame(n.nodes = as.numeric(names(tab)),
-	                  n.graphs = as.numeric(tab))
-	print(tab)
-	cat("\nPercent of vertices in the giant component:",
-	    round(100*vcount(ig1)/vcount(ig), 1), "%\n\n")
-
-	print(c(is.simple = is_simple(ig1),
-	        #is.connected = is_connected(ig1),
-	        is.dag = is_dag(ig1),
-	        is.directed = is_directed(ig1),
-	        is.weighted = is_weighted(ig1)))
-
-	cat("\n")
-	print(c(which.mutual = table(which_mutual(ig1))))
-
-	return(invisible(list(ig1 = ig1, gcs = gcs[-which.max(vsize)])))
-}
-
-mergeGraph <- function(g = list(), gref = NULL, gnet = NULL, verbose = FALSE,
-                       ...)
-{
-	g1 <- list()
-	for (i in 1:length(g)) {
-		V(g[[i]])$color <- NA
-		g[[i]] <- igraph::remove.vertex.attribute(g[[i]], "color")
-		if (!is_igraph(gnet) || !is_directed(gnet)) {
-			if (is.directed(g[[i]])) {
-				g1 <- c(g1, list(as.undirected(g[[i]], mode = "collapse")))
-			} else {
-				g1 <- c(g1, list(g[[i]]))
-			}
-		} else {
-			if (!is.directed(g[[i]])) {
-				g1 <- c(g1, list(orientEdges(graph = g[[i]], dg = gnet)))
-			} else {
-				g1 <- c(g1, list(g[[i]]))
-			}
-		}
-	}
-
-	Ug <- igraph::graph.union(g1)
-
-	if (is_igraph(gref)) {
-		V(Ug)$color <- ifelse(V(Ug)$name %in% V(gref)$name,
-		                      "lightblue",
-		                      "yellow")
-	} else {
-		V(Ug)$color <- "white"
-	}
-	if (verbose) plot(Ug)
-
-	return(Ug)
+    tab <- table(sapply(gcs, vcount))
+    tab <- data.frame(n.nodes = as.numeric(names(tab)), n.graphs = as.numeric(tab))
+    print(tab)
+    cat("\nPercent of vertices in the giant component:", 
+        round(100 * vcount(gcs[[1]])/vcount(ig), 1), "%\n\n")
+    print(c(is.simple = is_simple(gcs[[1]]), is.dag = is_dag(gcs[[1]]), 
+        is.directed = is_directed(gcs[[1]]), is.weighted = is_weighted(gcs[[1]])))
+    cat("\n")
+    print(c(which.mutual = table(which_mutual(gcs[[1]]))))
+    
+	return(gcs)
 }
 
 #' @title Graph plotting with renderGraph
@@ -833,7 +901,7 @@ orientEdges<- function(ug, dg, ...)
 
 #' @title Vertex and edge graph coloring on the base of fitting
 #'
-#' @description Add vertex and edge color atrributes to an igraph object,
+#' @description Add vertex and edge color attributes to an igraph object,
 #' based on a fitting results data.frame generated by
 #' \code{\link[SEMgraph]{SEMrun}}.
 #' @param est A data.frame of estimated parameters and p-values, derived
@@ -998,502 +1066,6 @@ colorMatch <- function(g1, g2, ...)
 	return(list(Vcol, Ecol, Ewid))
 }
 
-#' @title Optimal model search strategies
-#'
-#' @description Four model search strategies are implemented combining
-#' \code{SEMdag()}, \code{SEMbap()}, and \code{extendGraph()} functions.
-#' All strategies estimate a DAG through the adjusted (de-correlate)
-#' data matrix Z by iteratively update DAG and Z.
-#' @param graph Input network as an igraph object.
-#' @param data A matrix or data.frame. Rows correspond to subjects, and
-#' columns to graph nodes (variables).
-#' @param gnet Reference directed network used to validate and import
-#' nodes and interactions.
-#' @param search Search strategy. Four model search strategies are available:
-#' \itemize{
-#' \item "outer". The estimated DAG is extended using
-#' \code{\link[SEMgraph]{extendGraph}} to find new indirect paths (i.e.,
-#' inferred directed connections that may hide new mediators). New
-#' interactions and mediators will be searched and imported from the
-#' reference network (argument \code{gnet}, see above). Both DAG and
-#' extended graph complexity can be controlled with \code{beta} > 0 and
-#' \code{d} > 1 arguments, respectively (see below). The term "outer"
-#' means that new model mediator variables are imported from an external
-#' resource (i.e., the reference network).
-#' \item "inner". This strategy is analogous to the "outer" one,
-#' but disables external mediator search. In other words, new indirect
-#' paths are generated by adding new interactions of the input model, so
-#' that mediators will be nodes already present in the input graph. The
-#' reference network is still used to validate new model paths. Also in
-#' this case, \code{beta} > 0 and \code{d} > 1 are used.
-#' \item "direct". The input graph structure is improved through direct
-#' (i.e., adjacent) link search, followed by interaction validation and
-#' import from the reference network, with no mediators
-#' (i.e., \code{d = 1}).
-#' \item "basic" (default). While the previous strategies rely on the
-#' input graph and the reference network to integrate knowledge to the
-#' final model, the "basic" strategy is data-driven. The input graph is
-#' needed to define the topological order. The argument \code{gnet} is
-#' set to NULL (i.e., no reference network is needed) and argument
-#' \code{d = 0}. Model complexity can be still controlled by setting
-#' \code{beta} > 0.
-#' }
-#' @param beta Numeric value. Minimum absolute LASSO beta coefficient for
-#' a new interaction to be retained in the estimated DAG backbone. Lower
-#' \code{beta} values correspond to more complex DAGs.
-#' By default, \code{beta} is set to 0 (i.e., maximum complexity).
-#' @param d Maximum allowed geodesic distance for directed or undirected
-#' shortest path search. A distance \code{d = 0} disables shortest path
-#' search (fixed in \code{search = "basic"}), while \code{d = 1} 
-#' (fixed in \code{search = "direct"}) only search for directed links
-#' (i.e., no mediators are allowed).
-#' A distance \code{d} > 1 (defaults to \code{d = 2} for "outer" and
-#' "inner" strategies), will search for shortest paths with at most
-#' \code{d} - 1 mediators between nodes sharing a significant estimated
-#' interaction.
-#' Connectors are imported from the reference interactome, as specified
-#' by the argument gnet. If the edges of the reference interactome are
-#' weighted by P-value, as defined by the \code{E(graph)$pv} attribute,
-#' the shortest path with the smallest sum of weights will be chosen (e.g.,
-#' see \code{\link[SEMgraph]{weightGraph}} for graph weighting options).
-#' @param alpha Significance level for false discovery rate (FDR) used
-#' for either local d-separation tests (below \code{limit}) or conditional
-#' independence (CI) test (above \code{limit}). This argument is used to
-#' control data de-correlation. A higher \code{alpha} level includes more
-#' hidden covariances, thus considering more sources of confounding.
-#' If \code{alpha = 0}, data de-correlation is disabled.
-#' By default, \code{alpha = 0.05}.
-#' @param pstop A logical value. With the argument \code{pstop = TRUE}
-#' (default), the algorithm can be halted when the Shipley's global model
-#' test P-value > 0.05. If \code{pstop = FALSE}, the model search
-#' algorithm stops when no additional edges can be added to the estimated
-#' DAG.
-#' @param limit An integer value corresponding to the number of missing
-#' edges of the extracted acyclic graph. Beyond this limit, multicore
-#' computation is enabled to reduce the computational burden.
-#' @param verbose If TRUE, it shows intermediate graphs during the
-#' execution (not recommended for large graphs).
-#' @param ... Currently ignored.
-#'
-#' @details Search strategies can be ordered by decreasing conservativeness
-#' respect to the input graph, as: "direct", "inner", "outer", and "basic".
-#' The first three strategies are knowledge-based, since they require an
-#' input graph and a reference network, together with data, for
-#' knowledge-assisted model improvement. The last one does not require
-#' any reference and the output model structure will be completely
-#' determined by data.
-#' Output model complexity can be limited using arguments \code{d} and
-#' \code{beta}.
-#' While d is fixed to 0 or 1 in "basic" or "direct", respectively;
-#' we suggest starting with \code{d = 2} (only one mediator)
-#' for the other two strategies.
-#' For knowledge-based strategies, we suggest to to start with
-#' \code{beta = 0.1}. Then, beta can be relaxed (0 to < 0.1) to improve
-#' model fitting, if needed. Since data-driven models can be complex,
-#' we suggest to start from beta = 0.1 when using the "basic" strategy.
-#' The beta value can be relaxed until a good model fit is obtained.
-#' Argument alpha determines the extent of data adjustment: lower alpha
-#' values for FDR correction correspond to a smaller number of significant
-#' confounding factors, hence a weaker correction
-#' (default \code{alpha = 0.05}).
-#'
-#' @import lavaan
-#' @import igraph
-#' @import GGMncv
-#' @importFrom glmnet glmnet
-#' @importFrom RcppEigen fastLm
-#' @importFrom stats na.omit var cov qchisq pchisq p.adjust
-#' @importFrom corpcor is.positive.definite cor.shrink
-#' @importFrom flip flip plot
-#' @export
-#'
-#' @return The output model as well as the adjusted dataset are returned
-#' as a list of 3 objects:
-#' \itemize{
-#' \item "fit", the fitted output model (lavaan object);
-#' \item "graph", the output model as an igraph object;
-#' \item "data", the adjusted dataset.
-#' }
-#'
-#' @author Fernando Palluzzi \email{fernando.palluzzi@gmail.com}
-#'
-#' @examples
-#'
-#' \donttest{
-#'
-#' # Comparison among different model estimation strategies
-#'
-#' library(huge)
-#' als.npn <- huge.npn(alsData$exprs)
-#'
-#' # Models estimation
-#' m1 <- modelSearch(graph = alsData$graph, data = als.npn, gnet = kegg,
-#'       search = "direct", beta = 0, alpha = 0.05)
-#' m2 <- modelSearch(graph = alsData$graph, data = als.npn, gnet = kegg,
-#'       d = 2, search = "inner", beta = 0.05, alpha = 0.05)
-#' m3 <- modelSearch(graph = alsData$graph, data = als.npn, gnet = kegg,
-#'       d = 2, search = "outer", beta = 0.05, alpha = 0.05)
-#' m4 <- modelSearch(graph = alsData$graph, data = als.npn, gnet = NULL,
-#'       search = "basic", beta = 0.1, alpha = 0.05)
-#'
-#' # Graphs
-#' #old.par <- par(no.readonly = TRUE)
-#' #par(mfrow=c(2,2), mar= rep(1,4))
-#' gplot(m1$graph, main = "direct graph")
-#' gplot(m2$graph, main = "inner graph")
-#' gplot(m3$graph, main = "outer graph")
-#' gplot(m4$graph, main = "basic graph")
-#' #par(old.par)
-#'
-#' }
-#'
-modelSearch <- function(graph, data, gnet = NULL, d = 2, search = "basic",
-                        beta = 0, alpha = 0.05, pstop = TRUE, limit = 30000,
-                        verbose = FALSE, ...)
-{
-
-	# Set graph and data objects
-	nodes <- colnames(data)[colnames(data) %in% V(graph)$name]
-	ig <- induced_subgraph(graph, vids = which(V(graph)$name %in% nodes))
-	dataY <- as.matrix(data[, nodes])
-
-	# Stepwise search
-	if (search == "basic") d <- 0
-	if (search == "direct") d <- 1
-	Gt <- ig
-	Zt <- dataY
-	k <- 1
-
-	while (k > 0) {
-		Zt1 <- quiet(SEMbap(Gt, Zt, method = "BH", alpha = alpha,
-		                    limit = limit))
-
-		if (is.null(Zt1)) {
-			cat("k =", k, "Searching for missing covariances ... 0\n")
-			Zt1$data <- Zt
-			Gt1 <- quiet(SEMdag(Gt, Zt1$data, gnet = gnet, d = d,
-			                    beta = beta)$dag)
-			break
-		}
-
-		nE <- ecount(Zt1$guu)
-		cat("k =", k, "Searching for missing covariances ...", nE, "\n")
-		Gt1 <- quiet(SEMdag(Gt, Zt1$data, gnet = gnet, d = d, beta = beta)$dag)
-		if (pstop) {
-			ctest <- Shipley.test(Gt1, Zt1$data, limit = limit, verbose = FALSE)
-			pv <- ctest$ctest[3]
-		} else {
-			pv <- 0
-		}
-
-		if (pv > 0.05 | ecount(Gt1) - ecount(Gt) == 0) break
-		Gt <- Gt1
-		Zt <- Zt1$data
-		k <- k + 1
-	}
-	cat("Done.\n\n")
-
-	E1 <- attr(E(Gt1), "vnames")
-	E0 <- attr(E(ig), "vnames")
-	E(Gt1)$color <- ifelse(E1 %in% E0, "blue", "red")
-	Gt1.red <- Gt1 - E(Gt1)[which(E(Gt1)$color == "blue")]
-	if(ecount(Gt1.red) == 0) {
-		return(message("ERROR: no new edges inferred.
-		 Try decreasing the beta threshold !"))
-	}
-
-
-	# Extended graph
-
-	if (search == "basic") {
-		dataZ <- Zt1$data
-		ig <- Gt1
-
-	} else if (search != "basic") {
-
-		if (search == "direct" | search == "inner") {
-			ext <- extendGraph(g = list(Gt1,Gt1.red), Zt1$data,
-			                   gnet = gnet,
-			                   verbose = verbose)
-
-		} else if (search == "outer") {
-			ext <- extendGraph(g = list(Gt1, Gt1.red), data, gnet = gnet,
-							   verbose = verbose)
-		}
-
-		sel <- which(V(ext$Ug)$color == "yellow")
-		yellow <- names(V(ext$Ug)[sel])
-		new <- setdiff(yellow, V(ig)$name)
-		V(ext$Ug)$color[sel] <- ifelse(yellow %in% new, "yellow", "green")
-		dataZ <- cbind(Zt1$data, data[, new])
-		ig <- ext$Ug
-	}
-
-	if (verbose) gplot(ig, main = "Estimated Extended Graph")
-	if (!pstop) {
-		C_test <- Shipley.test(ig, dataZ, limit = limit, verbose = TRUE)
-	} else {
-		print(data.frame(C_test = ctest$ctest[1],
-		                 df = ctest$ctest[2],
-		                 pvalue = round(ctest$ctest[3], 6)))
-	}
-	cat("\n")
-	fit <- SEMrun(ig, dataZ, algo = "ricf")
-
-	return(list(fit = fit$fit, graph = ig, data = dataZ))
-}
-
-#' @title Perturbed path search utility
-#'
-#' @description This function uses \code{\link[SEMgraph]{SEMace}} to find
-#' significant causal effects between source-sink pairs and
-#' \code{\link[SEMgraph]{SEMpath}} to fit them and test their edge
-#' perturbation.
-#' @param graph Input network as an igraph object.
-#' @param data A matrix or data.frame. Rows correspond to subjects, and
-#' columns to graph nodes (variables).
-#' @param group group A binary vector. This vector must be as long as the
-#' number of subjects. Each vector element must be 1 for cases and 0
-#' for control subjects. Group specification enables edge perturbation
-#' testing. By default, \code{group = NULL}.
-#' @param ace A data.frame generated by \code{\link[SEMgraph]{SEMace}}.
-#' If NULL, \code{\link[SEMgraph]{SEMace}} will be automatically run.
-#' @param path If \code{path = "directed"}, all directed paths between
-#' the two nodes will be included in the fitted model.
-#' If \code{path = "shortest"}, only shortest paths will be considered.
-#' @param method Multiple testing correction method. One of the values
-#' available in \code{\link[stats]{p.adjust}}.
-#' By default, \code{method = "none"} (i.e., no multiple test correction).
-#' @param alpha Significance level for ACE selection (by default,
-#' \code{alpha = 0.05}).
-#' @param verbose Show the significant directed (or shortest) paths
-#' inside the input graph.
-#' @param ... Currently ignored.
-#'
-#' @import lavaan
-#' @import igraph
-#' @importFrom stats approx
-#' @importFrom dagitty paths
-#' @importFrom utils head
-#' @export
-#'
-#' @return A list of 3 objects:
-#' \itemize{
-#' \item "paths", list of paths as igraph objects;
-#' \item "fit", fitting results for each path as a lavaan object;
-#' \item "dfp", a data.frame containing SEM global fitting statistics.
-#' }
-#'
-#' @author Fernando Palluzzi \email{fernando.palluzzi@gmail.com}
-#'
-#' @examples
-#'
-#' \donttest{
-#'
-#' # Find and evaluate significantly perturbed paths
-#'
-#' library(huge)
-#' als.npn <- huge.npn(alsData$exprs)
-#'
-#' adjData <- SEMbap(graph = alsData$graph, data = als.npn)$data
-#'
-#' ace <- SEMace(graph = alsData$graph, data = adjData,
-#'               group = alsData$group)
-#' ace <- ace[order(ace$pvalue),]
-#' print(ace)
-#'
-#' paths <- pathFinder(graph = alsData$graph, data = adjData,
-#'                     group = alsData$group,
-#'                     ace = ace)
-#'
-#' head(parameterEstimates(paths$fit$P19))
-#' gplot(paths$paths$P19)
-#'
-#' path19 <- SEMpath(graph = alsData$graph, data = adjData,
-#'                   group = alsData$group,
-#'                   from = "7133",
-#'                   to = "4747",
-#'                   path = "directed",
-#'                   verbose = TRUE)
-#'
-#' }
-#'
-pathFinder <- function(graph, data, group = NULL, ace = NULL, path = "directed",
-                       method = "none", alpha = 0.05, verbose = FALSE, ...)
-{
-	if (is.null(ace)) {
-		ace <- SEMace(graph, data, group, method = method, alpha = alpha)
-	}
-	ace <- ace[ace$pvalue < alpha,]
-	ace <- ace[order(ace$pvalue),]
-	sources <- as.character(ace$source)
-	sinks <- as.character(ace$sink)
-	paths <- list()
-	lav <- list()
-	res <- NULL
-	N <- nrow(ace)
-	cat("\nFound", N, "significant ACEs.\n\n")
-	if (N == 0) return(list(paths = NULL, fit = NULL, dfp = NULL))
-	for (i in 1:N) {
-		fit <- quiet(SEMpath(graph, data, group, from = sources[i],
-		                     to = sinks[i],
-		                     path = path,
-		                     verbose = verbose))
-		if(is.null(fit)) next
-		if (!is.null(group) & vcount(fit$graph) > 100) {
-			dev_df <- fit$fit$ricf$dev/fit$fit$ricf$df
-			srmr <- fit$fit$fitIdx$srmr
-			pv1 <- Brown.test(x = fit$dataXY[, -1], p = fit$gest$pvalue,
-			                  theta = fit$gest$Stat,
-			                  tail = "positive")
-			pv2 <- Brown.test(x = fit$dataXY[, -1], p = fit$gest$pvalue,
-			                  theta = fit$gest$Stat,
-			                  tail = "negative")
-		} else {
-			dev_df <- fitMeasures(fit$fit, "chisq")/fitMeasures(fit$fit, "df")
-			srmr <- fitMeasures(fit$fit, "srmr")
-			pv1 <- Brown.test(x = fit$dataXY[, -1], p = fit$gest$pvalue,
-			                  theta = fit$gest$est,
-			                  tail = "positive")
-			pv2 <- Brown.test(x = fit$dataXY[, -1], p = fit$gest$pvalue,
-			                  theta = fit$gest$est,
-			                  tail = "negative")
-		}
-
-		dfp <- data.frame(pathId = paste0("P", rownames(ace)[i]),
-		                  sink = sinks[i],
-		                  op = "<-",
-		                  source = sources[i],
-		                  N.nodes = vcount(fit$graph),
-		                  N.edges = ecount(fit$graph),
-		                  dev_df = round(dev_df, 3),
-		                  srmr = round(srmr, 3),
-		                  pv.act = round(pv1, 6),
-		                  pv.inh = round(pv2, 6))
-
-		res <- rbind(res, dfp)
-		paths[[i]] <- fit$graph
-		lav[[i]] <- fit$fit
-	}
-	rownames(res) <- NULL
-	names(paths) <- res$pathId
-	names(lav) <- res$pathId
-	print(head(res))
-	return(list(paths = paths, fit = lav, dfp = res))
-}
-
-#' @title Cluster extraction utility
-#'
-#' @description Extract and fit clusters from an input graph.
-#'
-#' @param graph Input network as an igraph object.
-#' @param membership A vector of cluster membership IDs. If NULL, clusters
-#' will be automatically generated with \code{\link[SEMgraph]{clusterGraph}}
-#' using the edge betweenness clustering ("ebc") algorithm.
-#' @param data A matrix or data.frame. Rows correspond to subjects, and
-#' columns to graph nodes (variables).
-#' @param group A binary vector. This vector must be as long as the
-#' number of subjects. Each vector element must be 1 for cases and 0
-#' for control subjects. Group specification enables node perturbation
-#' testing. By default, \code{group = NULL}.
-#' @param map Logical value. If TRUE, the plot of the input graph
-#' (coloured by cluster membership) will be generated along with independent
-#' module plots. If the input graph is very large, plotting could be
-#' computationally intensive (by default, \code{map = FALSE}).
-#' @param verbose Logical value. If TRUE, a plot will be showed for each
-#' cluster.
-#' @param ... Currently ignored.
-#'
-#' @import igraph
-#' @import lavaan
-#' @importFrom stats cor approx
-#' @importFrom corpcor is.positive.definite cor.shrink
-#' @importFrom ggm fitAncestralGraph
-#' @importFrom flip flip plot
-#' @export
-#'
-#' @return List of clusters as igraph objects and fitting results for
-#' each cluster as a lavaan object.
-#'
-#' @author Fernando Palluzzi \email{fernando.palluzzi@gmail.com}
-#'
-#' @examples
-#'
-#' \donttest{
-#'
-#' library(huge)
-#' als.npn <- huge.npn(alsData$exprs)
-#'
-#' adjdata <- SEMbap(alsData$graph, als.npn)$data
-#'
-#' # Clusters creation
-#' clusters <- extractClusters(graph = alsData$graph, data = adjdata)
-#' head(parameterEstimates(clusters$fit$HM1))
-#' head(parameterEstimates(clusters$fit$HM2))
-#' head(parameterEstimates(clusters$fit$HM4))
-#' gplot(clusters$clusters$HM2)
-#'
-#' # Map cluster on the input graph
-#' g <- alsData$graph
-#' c <- clusters$clusters$HM2
-#' V(g)$color <- ifelse(V(g)$name %in% V(c)$name, "gold", "white")
-#' gplot(g)
-#'
-#' }
-#'
-extractClusters <- function(graph, data, group = NULL, membership = NULL,
-                            map = FALSE, verbose = FALSE, ...)
-{
-	if (is.null(membership)) {
-		membership <- clusterGraph(graph, type = "ebc", HM = "none",
-		                           size = 5,
-		                           verbose = FALSE)
-	}
-	clusters <- cplot(graph, membership, l=layout.auto, map, verbose)[-1]
-	if ("HM9999" %in% names(clusters)) N <- length(clusters) - 1
-	N <- length(clusters)
-	res <- NULL
-	lav <- list()
-	for (i in 1:N) {
-		fit <- quiet(SEMfit(clusters[[i]], data, group))
-		if (is.null(fit)) next
-		if (!is.null(group) & vcount(clusters[[i]]) > 100) {
-			dev_df <- fit$fit$ricf$dev/fit$fit$ricf$df
-			srmr <- fit$fit$fitIdx[3]
-			pv1 <- Brown.test(x = fit$dataXY[, -1], p = fit$gest$pvalue,
-			                  theta = fit$gest$Stat,
-			                  tail = "positive")
-			pv2 <- Brown.test(x = fit$dataXY[, -1], p = fit$gest$pvalue,
-			                  theta = fit$gest$Stat,
-			                  tail = "negative")
-		} else {
-			dev_df <- fitMeasures(fit$fit, "chisq")/fitMeasures(fit$fit, "df")
-			srmr <- fitMeasures(fit$fit, "srmr")
-			pv1 <- Brown.test(x = fit$dataXY[, -1], p = fit$gest$pvalue,
-			                  theta = fit$gest$est,
-			                  tail = "positive")
-			pv2 <- Brown.test(x = fit$dataXY[, -1], p = fit$gest$pvalue,
-			                  theta = fit$gest$est,
-			                  tail = "negative")
-		}
-
-		dfc <- data.frame(cluster = names(clusters)[i],
-		                  N.nodes = vcount(clusters[[i]]),
-		                  N.edges = ecount(clusters[[i]]),
-		                  dev_df = round(dev_df, 3),
-		                  srmr = round(srmr, 3),
-		                  pv.act = round(pv1, 6),
-		                  pv.inh = round(pv2, 6))
-
-		res <- rbind(res, dfc)
-		lav[[i]] <- fit$fit
-	}
-	rownames(res) <- NULL
-	names(lav) <- names(clusters)
-	print(res)
-	return(list(clusters = clusters, fit = lav, dfc = res))
-}
-
 #' @title Pairwise plotting of multivariate data
 #'
 #' @description Display a pairwise scatter plot of two datasets for a
